@@ -30,6 +30,7 @@ import wizard
 import printer
 import common
 import tools
+import options
 from widget.view.form_gtk.many2one import dialog
 
 class main(service.Service):
@@ -51,6 +52,7 @@ class main(service.Service):
         report_id = rpc.session.rpc_exec_auth('/report', 'report', name, ids, datas, ctx)
         state = False
         attempt = 0
+        max_attemps = int(options.options.get('client.timeout') or 0)
         while not state:
             val = rpc.session.rpc_exec_auth('/report', 'report_get', report_id)
             if not val:
@@ -59,7 +61,7 @@ class main(service.Service):
             if not state:
                 time.sleep(1)
                 attempt += 1
-            if attempt>200:
+            if attempt>max_attemps:
                 common.message(_('Printing aborted, too long delay !'))
                 return False
         printer.print_data(val)
@@ -81,6 +83,8 @@ class main(service.Service):
     def _exec_action(self, action, datas, context={}):
         if isinstance(action, bool) or 'type' not in action:
             return
+        # Update context, adding the dynamic context of the action
+        context.update(tools.expr_eval(action.get('context','{}'), context.copy()))
         if action['type']=='ir.actions.act_window':
             for key in ('res_id', 'res_model', 'view_type', 'view_mode',
                     'limit', 'auto_refresh'):
@@ -132,7 +136,10 @@ class main(service.Service):
             ctx.update({'active_id': datas.get('id',False), 'active_ids': datas.get('ids',[])})
             res = rpc.session.rpc_exec_auth('/object', 'execute', 'ir.actions.server', 'run', [action['id']], ctx)
             if res:
-                self._exec_action(res, datas, context)
+                if not isinstance(res, list):
+                    res = [res]
+                for r in res:
+                    self._exec_action(r, datas, context)
 
         elif action['type']=='ir.actions.wizard':
             win=None
@@ -145,6 +152,7 @@ class main(service.Service):
             if 'window' in datas:
                 win=datas['window']
                 del datas['window']
+            datas.update(action.get('datas',{}))
             datas['report_id'] = action['report_id']
             self.exec_report('custom', datas, context)
 
@@ -152,6 +160,7 @@ class main(service.Service):
             if 'window' in datas:
                 win=datas['window']
                 del datas['window']
+            datas.update(action.get('datas',{}))
             self.exec_report(action['report_name'], datas, context)
 
         elif action['type']=='ir.actions.act_url':
