@@ -32,6 +32,7 @@ import widget_search
 import signal_event
 import tools
 import service
+import copy
 
 
 class Screen(signal_event.signal_event):
@@ -90,6 +91,7 @@ class Screen(signal_event.signal_event):
         self.tree_saves = tree_saves
         self.limit = limit
         self.readonly= readonly
+        self.view_fields = {} # Used to switch self.fields when the view switchs
 
         if view_type:
             self.view_to_load = view_type[1:]
@@ -124,9 +126,15 @@ class Screen(signal_event.signal_event):
                 child_node=dom.childNodes[0].childNodes
                 arch=''
                 for i in range(1,len(child_node)):
-                    arch+=child_node[i].toxml()
-                view_form['arch']=view_form['arch'][0:len(view_form['arch'])-7]+arch
-                view_form['arch']=view_form['arch']+'\n</form>'
+                    arch += child_node[i].toxml()
+                #Generic case when we need to remove the last occurance of </form> from form view    
+                view_form['arch'] = view_form['arch'][0:view_form['arch'].rfind('</form>')]
+                #Special case when form is replaced,we need to remove </form>
+                find_form = view_form['arch'].rfind('</form>')
+                if find_form > 0:
+                    view_form['arch'] = view_form['arch'][0:find_form]
+
+                view_form['arch'] = view_form['arch'] + arch + '\n</form>'
                 view_form['fields'].update(view_tree['fields'])
                 self.filter_widget = widget_search.form(view_form['arch'],
                         view_form['fields'], self.name, self.window,
@@ -136,7 +144,7 @@ class Screen(signal_event.signal_event):
                         self.search_offset_next,
                         self.search_offset_previous)
                 self.filter_widget.set_limit(self.limit)
-
+                
         if active and show_search:
             self.screen_container.show_filter()
         else:
@@ -225,8 +233,11 @@ class Screen(signal_event.signal_event):
             view.reload = True
 
     def _record_changed(self, model_group, signal, *args):
-        for view in self.views:
-            view.signal_record_changed(signal[0], model_group.models, signal[1], *args)
+        try:
+            for view in self.views:
+                view.signal_record_changed(signal[0], model_group.models, signal[1], *args)
+        except:
+            pass        
 
     def _model_changed(self, model_group, model):
         if (not model) or (model==self.current_model):
@@ -267,6 +278,7 @@ class Screen(signal_event.signal_event):
     # mode: False = next view, value = open this view
     def switch_view(self, screen=None, mode=False):
         self.current_view.set_value()
+        self.fields = {}
         if self.current_model and self.current_model not in self.models.models:
             self.current_model = None
         if mode:
@@ -293,6 +305,11 @@ class Screen(signal_event.signal_event):
                 self.__current_view = len(self.views) - 1
             else:
                 self.__current_view = (self.__current_view + 1) % len(self.views)
+        
+        self.fields = self.view_fields.get(self.__current_view, self.fields) # Switch the fields
+        # TODO: maybe add_fields_custom is needed instead of add_fields on some cases
+        self.models.add_fields(self.fields, self.models) # Switch the model fields too
+        
         widget = self.current_view.widget
         self.screen_container.set(self.current_view.widget)
         if self.current_model:
@@ -345,7 +362,7 @@ class Screen(signal_event.signal_event):
                         if attrs['widget']=='one2many_list':
                             attrs['widget']='one2many'
                         attrs['type'] = attrs['widget']
-                    fields[str(attrs['name'])].update(attrs)
+                    fields[unicode(attrs['name'])].update(attrs)
             for node2 in node.childNodes:
                 _parse_fields(node2, fields)
         dom = xml.dom.minidom.parseString(arch)
@@ -377,6 +394,10 @@ class Screen(signal_event.signal_event):
             self.__current_view = len(self.views) - 1
             self.current_view.display()
             self.screen_container.set(view.widget)
+        
+        # Store the fields for this view (we will use them when switching views)
+        self.view_fields[len(self.views)-1] = copy.deepcopy(self.fields)
+
         return view
 
     def editable_get(self):
@@ -529,6 +550,8 @@ class Screen(signal_event.signal_event):
         return id
 
     def load(self, ids):
+        if not isinstance(ids,list):
+            ids = [ids]
         self.models.load(ids, display=False)
         self.current_view.reset()
         if ids:
