@@ -68,7 +68,14 @@ class float_time(interface.widget_interface):
             return False
         super(float_time, self).display(model, model_field)
         val = model_field.get(model)
-        t = '%02d:%02d' % (math.floor(abs(val)),round(abs(val)%1+0.01,4) * 60)
+        hours = math.floor(abs(val))
+        mins = round(abs(val)%1+0.01,2)
+        if mins >= 1.0:
+            hours = hours + 1
+            mins = 0.0
+        else:
+            mins = mins * 60
+        t = '%02d:%02d' % (hours,mins)
         if val<0:
             t = '-'+t
         self.widget.set_text(t)
