@@ -78,16 +78,34 @@ class Button(Observable):
         model = self.form.screen.current_model
         self.form.set_value()
         button_type = self.attrs.get('special', '')
+
         if button_type=='cancel':
             self.form.screen.window.destroy()
             if 'name' in self.attrs.keys():
-                result = rpc.session.rpc_exec_auth(
-                            '/object', 'execute',
-                            self.form.screen.name,
-                            self.attrs['name'],[], model.context_get())
-                datas = {}
-                obj = service.LocalService('action.main')
-                obj._exec_action(result,datas,context=self.form.screen.context)
+                type_button = self.attrs.get('type','object')
+
+                if type_button == 'action':
+                    obj = service.LocalService('action.main')
+                    action_id = int(self.attrs['name'])
+
+                    context_action = self.form.screen.context.copy()
+                    
+                    if 'context' in self.attrs:
+                        context_action.update(self.form.screen.current_model.expr_eval(self.attrs['context'], check_load=False))
+
+                    obj.execute(action_id, {'model':self.form.screen.name, 'id': False, 'ids': [], 'report_type': 'pdf'}, context=context_action)
+
+                elif type_button == 'object':
+                    result = rpc.session.rpc_exec_auth(
+                                '/object', 'execute',
+                                self.form.screen.name,
+                                self.attrs['name'],[], model.context_get())
+                    datas = {}
+                    obj = service.LocalService('action.main')
+                    obj._exec_action(result,datas,context=self.form.screen.context)
+                else:
+                    raise Exception, 'Unallowed button type'
+                
         elif model.validate():
             id = self.form.screen.save_current()
             if not self.attrs.get('confirm',False) or \
@@ -105,34 +123,45 @@ class Button(Observable):
                             obj = service.LocalService('action.main')
                             obj._exec_action(result, datas)
                     elif type([]) == type(result):
-                        datas = {'ids':[id]}
+                        datas = {'ids' : [id], 'model' : self.form.screen.name, }
                         obj = service.LocalService('action.main')
                         for rs in result:
                             obj._exec_action(rs, datas)
-                            
+
                 elif button_type == 'object':
                     if not id:
                         return
+                    context = model.context_get()
+                    if 'context' in self.attrs:
+                        context.update(self.form.screen.current_model.expr_eval(self.attrs['context'], check_load=False))
+
                     result = rpc.session.rpc_exec_auth(
                         '/object', 'execute',
                         self.form.screen.name,
                         self.attrs['name'],
-                        [id], model.context_get()
+                        [id], context
                     )
                     if type(result)==type({}):
                         self.form.screen.window.destroy()
-                        datas = {}
+                        datas = {'ids' : [id], 'model' : self.form.screen.name, }
                         obj = service.LocalService('action.main')
                         obj._exec_action(result,datas,context=self.form.screen.context)
 
                 elif button_type == 'action':
                     obj = service.LocalService('action.main')
                     action_id = int(self.attrs['name'])
-                    obj.execute(action_id, {'model':self.form.screen.name, 'id': id or False,
-                        'ids': id and [id] or [], 'report_type': 'pdf'}, context=self.form.screen.context)
+
+                    context = self.form.screen.context.copy()
+
+                    if 'context' in self.attrs:
+                        context.update(self.form.screen.current_model.expr_eval(self.attrs['context'], check_load=False))
+
+                    obj.execute(action_id, {'model':self.form.screen.name, 'id': id or False, 'ids': id and [id] or [], 'report_type': 'pdf'}, context=context)
+
                 else:
                     raise Exception, 'Unallowed button type'
                 self.form.screen.reload()
+                self.warn('misc-message', '')
         else:
             common.warning(_('Invalid form, correct red fields !'), _('Error !') )
             self.warn('misc-message', _('Invalid form, correct red fields !'), "red")
@@ -177,11 +206,10 @@ class StateAwareWidget(object):
 
 
 class _container(object):
-    def __init__(self, tooltips):
+    def __init__(self):
         self.cont = []
         self.col = []
         self.sg = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        self.tooltips = tooltips
         self.trans_box = []
         self.trans_box_label = []
 
@@ -209,7 +237,7 @@ class _container(object):
         table.resize(y+1,self.col[-1])
 
     def wid_add(self, widget, name=None, expand=False, ypadding=2, rowspan=1,
-            colspan=1, translate=False, fname=None, help=False, fill=False):
+            colspan=1, translate=False, fname=None, help=False, fill=False, invisible=False):
         (table, x, y) = self.cont[-1]
         if colspan>self.col[-1]:
             colspan=self.col[-1]
@@ -263,18 +291,16 @@ class _container(object):
         wid_list = table.get_children()
         wid_list.reverse()
         table.set_focus_chain(wid_list)
+        if invisible:
+            hbox.hide()
 
 class parser_form(widget.view.interface.parser_interface):
-    def parse(self, model, root_node, fields, notebook=None, paned=None, tooltips=None):
+    def parse(self, model, root_node, fields, notebook=None, paned=None):
         dict_widget = {}
         saw_list = []   # state aware widget list
         attrs = tools.node_attributes(root_node)
         on_write = attrs.get('on_write', '')
-        if not tooltips:
-            self.tooltips = gtk.Tooltips()
-        else:
-            self.tooltips = tooltips
-        container = _container(self.tooltips)
+        container = _container()
         container.new(col=int(attrs.get('col', 4)))
         self.container = container
 
@@ -370,7 +396,7 @@ class parser_form(widget.view.interface.parser_interface):
                 nb.set_tab_pos(pos)
                 nb.set_border_width(3)
                 container.wid_add(nb, colspan=attrs.get('colspan', 3), expand=True, fill=True )
-                _, widgets, saws, on_write = self.parse(model, node, fields, nb, tooltips=self.tooltips)
+                _, widgets, saws, on_write = self.parse(model, node, fields, nb)
                 saw_list += saws
                 dict_widget.update(widgets)
 
@@ -386,7 +412,7 @@ class parser_form(widget.view.interface.parser_interface):
                 l = gtk.Label(attrs.get('string','No String Attr.'))
                 l.attrs=attrs.copy()
                 l.set_angle(angle)
-                widget, widgets, saws, on_write = self.parse(model, node, fields, notebook, tooltips=self.tooltips)
+                widget, widgets, saws, on_write = self.parse(model, node, fields, notebook)
                 saw_list += saws
                 dict_widget.update(widgets)
                 notebook.append_page(widget, l)
@@ -394,15 +420,12 @@ class parser_form(widget.view.interface.parser_interface):
             elif node.localName=='field':
                 name = str(attrs['name'])
                 del attrs['name']
+                name = unicode(name)
                 type = attrs.get('widget', fields[name]['type'])
                 fields[name].update(attrs)
                 fields[name]['model']=model
                 if not type in widgets_type:
                     continue
-                if attrs.get('invisible', False):
-                    visval = eval(attrs['invisible'], {'context':self.screen.context})
-                    if visval:
-                        continue
 
                 fields[name]['name'] = name
                 if 'saves' in attrs:
@@ -427,6 +450,10 @@ class parser_form(widget.view.interface.parser_interface):
                 if attrs.get('height', False) or attrs.get('width', False):
                     widget_act.widget.set_size_request(
                             int(attrs.get('width', -1)), int(attrs.get('height', -1)))
+                if attrs.get('invisible', False):
+                    visval = eval(attrs['invisible'], {'context':self.screen.context})
+                    if visval:
+                        continue
                 container.wid_add(widget_act.widget, label, expand, translate=fields[name].get('translate',False), colspan=size, fname=name, help=hlp, fill=fill)
 
             elif node.localName=='group':
@@ -443,7 +470,7 @@ class parser_form(widget.view.interface.parser_interface):
                 container.wid_add(frame, colspan=int(attrs.get('colspan', 1)), expand=int(attrs.get('expand',0)), rowspan=int(attrs.get('rowspan', 1)), ypadding=0, fill=int(attrs.get('fill', 1)))
                 container.new(int(attrs.get('col',4)))
 
-                widget, widgets, saws, on_write = self.parse(model, node, fields, tooltips=self.tooltips)
+                widget, widgets, saws, on_write = self.parse(model, node, fields)
                 dict_widget.update(widgets)
                 saw_list += saws
                 frame.add(widget)
@@ -454,7 +481,7 @@ class parser_form(widget.view.interface.parser_interface):
             elif node.localName=='hpaned':
                 hp = gtk.HPaned()
                 container.wid_add(hp, colspan=int(attrs.get('colspan', 4)), expand=True, fill=True)
-                _, widgets, saws, on_write = self.parse(model, node, fields, paned=hp, tooltips=self.tooltips)
+                _, widgets, saws, on_write = self.parse(model, node, fields, paned=hp)
                 saw_list += saws
                 dict_widget.update(widgets)
                 #if 'position' in attrs:
@@ -462,18 +489,18 @@ class parser_form(widget.view.interface.parser_interface):
             elif node.localName=='vpaned':
                 hp = gtk.VPaned()
                 container.wid_add(hp, colspan=int(attrs.get('colspan', 4)), expand=True, fill=True)
-                _, widgets, saws, on_write = self.parse(model, node, fields, paned=hp, tooltips=self.tooltips)
+                _, widgets, saws, on_write = self.parse(model, node, fields, paned=hp)
                 saw_list += saws
                 dict_widget.update(widgets)
                 if 'position' in attrs:
                     hp.set_position(int(attrs['position']))
             elif node.localName=='child1':
-                widget, widgets, saws, on_write = self.parse(model, node, fields, paned=paned, tooltips=self.tooltips)
+                widget, widgets, saws, on_write = self.parse(model, node, fields, paned=paned)
                 saw_list += saws
                 dict_widget.update(widgets)
                 paned.pack1(widget, resize=True, shrink=True)
             elif node.localName=='child2':
-                widget, widgets, saws, on_write = self.parse(model, node, fields, paned=paned, tooltips=self.tooltips)
+                widget, widgets, saws, on_write = self.parse(model, node, fields, paned=paned)
                 saw_list += saws
                 dict_widget.update(widgets)
                 paned.pack2(widget, resize=True, shrink=True)
